"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
function fixControlledValue(value) {
    if (typeof value === 'undefined' || value === null) {
        return '';
    }
    return value;
}
/**
 * 百度UEditor的React封装
 */
var UEditor = /** @class */ (function (_super) {
    __extends(UEditor, _super);
    function UEditor(props) {
        return _super.call(this, props) || this;
    }
    /**
     *
     * @param nextProps
     */
    UEditor.prototype.componentWillReceiveProps = function (nextProps) {
        if (UE && UE.getEditor) {
            // 只有在受控模式下，才会试图同步编辑器的值
            if ('value' in nextProps) {
                var nextValue = fixControlledValue(nextProps.value);
                var thisValue = fixControlledValue(this.props.value);
                // 如果下一个value值和现在的value值相同，则不再同步。
                if (nextValue == thisValue) {
                    return;
                }
                var ue = UE.getEditor(this.props.id, {
                    initialFrameWidth: this.props.width,
                    initialFrameHeight: this.props.height,
                });
                ue.setContent(nextValue);
            }
        }
        else {
            console.log("error happpens: \u8BD5\u56FE\u8BBE\u7F6Evalue\u5C5E\u6027\uFF0C\u4F46\u662FUE.getEditor\u4E0D\u53EF\u7528");
        }
    };
    UEditor.prototype.componentWillMount = function () {
        if (typeof UE != 'undefined' && !!UE.getEditor && !!UE.delEditor) {
            // 如果UE已经是全局变量了，则说明已经加载了UEditor相应的<script>
            return;
        }
        else {
            var scriptConfig = document.querySelector("script[src='" + this.props.uconfigSrc + "']");
            if (!scriptConfig) {
                scriptConfig = document.createElement("script");
                scriptConfig.src = this.props.uconfigSrc;
                document.body.appendChild(scriptConfig);
            }
            var scriptEditor = document.querySelector("script[src='" + this.props.ueditorSrc + "']");
            if (!scriptEditor) {
                scriptEditor = document.createElement("script");
                scriptEditor.src = this.props.ueditorSrc;
                document.body.appendChild(scriptEditor);
            }
        }
    };
    UEditor.prototype.componentDidMount = function () {
        var _a = this.props, id = _a.id, width = _a.width, height = _a.height, initialContent = _a.initialContent, value = _a.value, afterInit = _a.afterInit, onChange = _a.onChange;
        if ('value' in this.props) {
            value = fixControlledValue(value);
            initialContent = value;
        }
        else {
            initialContent = fixControlledValue(initialContent);
        }
        function timeoutPromise(timeout) {
            return new Promise(function (resolve, reject) {
                setTimeout(function () {
                    resolve();
                }, timeout);
            });
        }
        function waitUntil() {
            return new Promise(function (resolve, reject) {
                var ue = UE.getEditor(id, {
                    initialFrameWidth: width,
                    initialFrameHeight: height,
                });
                ue.setDisabled();
                ue.ready(function () {
                    ue.setContent(initialContent);
                    ue.setEnabled();
                    // 触发 afterInit() 回调
                    afterInit(ue);
                    // 监听UE的 contentChange 事件
                    ue.addListener('contentChange', function (type) {
                        var content = ue.getContent();
                        // 触发 onChange()  回调
                        onChange(content);
                    });
                    resolve(ue);
                });
            }).catch(function (err) {
                console.log("the UE has not been ready yet. waitting 30ms ...", err);
                return timeoutPromise(30).then(function () {
                    return waitUntil();
                });
            });
        }
        return waitUntil();
    };
    UEditor.prototype.componentWillUnmount = function () {
        UE.delEditor(this.props.id);
        // 不再卸载<script> :
        //     <script type="text/javascript" src="/static/ueditor/ueditor.config.js"></script>
        //     <script type="text/javascript" src="/static/ueditor/ueditor.all.min.js"></script>
    };
    UEditor.prototype.render = function () {
        return (React.createElement("script", { id: this.props.id, type: "text/plain" }));
    };
    UEditor.defaultProps = {
        id: 'ueditorcontainer',
        height: 600,
        width: 600,
        uconfigSrc: "/static/ueditor/ueditor.config.js",
        ueditorSrc: "/static/ueditor/ueditor.all.min.js",
        afterInit: function (ue) { },
        onChange: function (content) { },
    };
    return UEditor;
}(React.Component));
exports.UEditor = UEditor;
exports.default = UEditor;
//# sourceMappingURL=index.js.map